// Demo 12: Parabola.java

//      The Area Under A Parabola
//      A Screen Based Application
//
//      by Charles W. Neville, 1998
//   Copyright Charles W. Neville, 1998


// Illustrates for loops and if - else.
// Illustrates use of double precision floating point numbers.
// Illustrates use of class Random in simulations.

// Also illustrate an important class of methods for scientific computing, 
// Monte Carlo methods.  In a Monte Carlo method, one uses a random process 
// (pseudo-random process actually) to simulate something in nature or 
// mathematics, usually something quite complex.  Monte Carlo methods are named 
// after the famous casino in Monte Carlo.

// Here is how this program works.  We wish to compute an approximation to the 
// area of the plane region bounded by the parabola y = 1 - x*x. and the x axis 
// (the area under the parabola).  We do this in the following manner:
// Simulate throwing 10,000 darts at rectangle of width 2 and height 1 with 
// vertices (corners) at (-1, 0), (1, 0), (1, 1) and (-1, 1).  Note that you 
// could draw the region under the parabola and have it just touch the edges of 
// the rectangle (be inscribed in the rectangle).  Count the number of darts 
// which hit the parabolic region, and at the end, compute the fraction of 
// darts which hit the parabolic region by dividing the total number of hits by 
// 10,000.  Now the fraction f of darts hitting the parabolic region is equal 
// to

//     areaOfParabolicRegion/AreaOfRectangle,

// except for statistical error.  Now areaOfRectangle = 2*1 = 2.  Thus the 
// fraction f is equal to areaOfParabolicRegion/2, except for statistical 
// error.  Multiply f by 2 to get an approximation to the area of the parabolic 
// region,

//     areaOfParabolicRegion = 2*f

// approximately.  (Approximately because of the statistical error.)

// When I ran this program 4 consecutive times, I got the following approximate 
// values for the area of the parabolic region: 1.3298, 1.3316, 1.3406, and 
// 1.3340.  The exact value (from calculus) is 4/3 = 1.33333333... .  The 
// approximation is pretty good, don't you think?

// By the way, very well paid people on Wall Street actually evaluate very 
// complicated definite integrals in this very way.

// there is no keyboard input to this program, so we do not need to import
// java.io.

import java.util.*;  // have to import java.util.* because Java needs to
                     // know what class Random is.

class Parabola {

   public static void main( String args[] ) {

     // this is a screen based application, so we don't need to construct
     // an instance of ourselves, PROVIDED that all methods are declared
     // as static, and all main program variables are defined inside method
     // main (or declared as static).

     final int numTimes = 10000;  // a constant declaration.

     int numHits = 0;  // number of hits in parabolic region.

     double x, y;  // coordinates of random point in rectangle.

     double f;  // fraction of times dart hit parabolic region

     double areaOfParabolicRegion;  // obviously, the area of the 
                                    // parabolic region.

     Random random = new Random();  // declare and create a new Random object.

     // our first for loop.  Go through this loop exactly numTimes times
     // (exactly 10,000 times).

     for( int i = 1; i <= numTimes; i++ ) {

         // Throw the dart at rectangle by randomly generating x and y values
         // between 0.0 and 1.0
         x = random.nextDouble();
         y = random.nextDouble();

         // But want random values for x to be between -1.0 and 1.0
         x = -1 + 2*x;

         // check to see if dart hit parabolic region, and if so,
         // update the number of hits
         if ( y <= 1 - x*x ) {
            numHits = numHits + 1;
         }
     }

     // what fraction of the darts hit the parabolic region?
     //   careful! Convert numHits to a double first because / means
     //            means INTEGER DIVISION when only integers are involved.
     f = (double) numHits / numTimes;

          // Note: The above (double) numHits is our first cast.  Casting
          //       means changing one type to another compatible type.

     // what is the area of the parabolic region?
     areaOfParabolicRegion = 2*f;

     System.out.println( "The area of the parabolic region is: " + 
areaOfParabolicRegion );
   }
}

